CLF_tree = "DMG"

function CritlineFilters_OnLoad(self)
	self:RegisterEvent("ADDON_LOADED")

	CLF_Pre_SpellButton_OnModifiedClick = SpellButton_OnModifiedClick
	SpellButton_OnModifiedClick = CLF_SpellButton_OnModifiedClick

	CritlineFilters_Localize()
	CritlineFilters_TreeInitialize()
end

function CLF_SpellButton_OnModifiedClick(self,drag)
	if IsShiftKeyDown() then
		if Critline_Filters:IsVisible() then
			local spellId = SpellBook_GetSpellID(self:GetID())
			if spellId > MAX_SPELLS then
				return
			end
			local spellName, spellRank = GetSpellName(spellId, SpellBookFrame.bookType)
			CritlineFilters_AddSpell(spellName, CLF_tree)
			return
		end
    end
    CLF_Pre_SpellButton_OnModifiedClick(self, drag)
end

function CritlineFilters_ShowFilters()
	Critline_Filters:Show()
	CritlineSettings_Frame:Hide()
end

function CritlineFilters_AddTarget()
	local targetName = UnitName("target")
	if targetName then
		if not CritlineFilters_IsMobInFilter(targetName) then
			tinsert(CL_MOBFILTERS["mobs"], targetName)
			CLFMobScrollBar_Update()
			Critline_Message(targetName.." added to mob filter.")
		end
	end
end

function CLFMobScrollBar_Update()
	local lineplusoffset -- an index into our data calculated from the scroll offset
	local size
	if CL_MOBFILTERS["mobs"] then
		size = #CL_MOBFILTERS["mobs"]
	end
	if not size then
		size = 0
	end
	
	local button
	
	FauxScrollFrame_Update(CLFMobScrollBar, size, 10, 16)
	for line = 1, 10 do
		button = _G["CLFMobButton"..line]
		lineplusoffset = line + FauxScrollFrame_GetOffset(CLFMobScrollBar)
		if lineplusoffset <= size then
			button:SetText(CL_MOBFILTERS["mobs"][lineplusoffset])
			button:Show()
		else
			button:Hide()
		end
	end
end

function CLFSpellScrollBar_Update()
	local lineplusoffset -- an index into our data calculated from the scroll offset
	local CL_SPELLS = {}

	if CL_FILTERS[CLF_tree] then
		for k, v in pairs(CL_FILTERS[CLF_tree]) do
			tinsert(CL_SPELLS, v)
		end
		for k, v in pairs(CL_DATABASE[CLF_tree]) do
			if type(k) == "number" then
				CL_DATABASE[CLF_tree][k] = nil -- need to get rid of old 113 errors...they are stored as numbers and cause errors when compared with string names
			elseif not CritlineFilters_IsSpellInFilter(k, CLF_tree) then
				tinsert(CL_SPELLS, k)
			end
		end
	end

	sort(CL_SPELLS)
	local size = #CL_SPELLS
	local buttonName, button
	
	FauxScrollFrame_Update(CLFSpellScrollBar, size, 7, 16)
	for line = 1, 7 do
		buttonName = "CLFSpellButton"..line
		button = _G[buttonName]
		lineplusoffset = line + FauxScrollFrame_GetOffset(CLFSpellScrollBar)
		if lineplusoffset <= size then
			_G[buttonName.."Text"]:SetText(CL_SPELLS[lineplusoffset])
			if CritlineFilters_IsSpellInFilter(CL_SPELLS[lineplusoffset], CLF_tree) then
				button:SetChecked(true)
			else
				button:SetChecked(false)
			end
			button:Show()
		else
			button:Hide()
		end
	end
end
 
function CritlineFilters_TreeInitialize()
	UIDropDownMenu_Initialize(Critline_FiltersTree, function()
		local tex
		tex = {
			text = DAMAGE,
			value = 1,
			func = function()
				UIDropDownMenu_SetSelectedValue(Critline_FiltersTree, 1)
				CLA_SelectedSpells = {}
				FauxScrollFrame_SetOffset(CLFSpellScrollBar, 0)
				CLF_tree = "DMG"
				CLFSpellScrollBar_Update()
			end
		}
		UIDropDownMenu_AddButton(tex)
		
		tex = {
			text = HEALS,
			value = 2,
			func = function()
				UIDropDownMenu_SetSelectedValue(Critline_FiltersTree, 2)
				CLA_SelectedSpells = {}
				FauxScrollFrame_SetOffset(CLFSpellScrollBar, 0)
				CLF_tree = "HEAL"
				CLFSpellScrollBar_Update()
			end
		}
		UIDropDownMenu_AddButton(tex)
		
		tex = {
			text = PET,
			value = 3,
			func = function()
				UIDropDownMenu_SetSelectedValue(Critline_FiltersTree, 3)
				CLF_SelectedSpells = {}
				FauxScrollFrame_SetOffset(CLFSpellScrollBar, 0)
				CLF_tree = "PET"
				CLFSpellScrollBar_Update()
			end
		}
		UIDropDownMenu_AddButton(tex)
	end)
	UIDropDownMenu_SetWidth(Critline_FiltersTree, 120)
end

function CritlineFilters_DeleteTarget()
	if CLFMobScrollBar.selected then
		tremove(CL_MOBFILTERS["mobs"], CLFMobScrollBar.selected)
		CLFMobScrollBar.selected = nil
		CLFMobScrollBar_Update()
		Critline_Message(targetName.." removed from mob filter.")
	end
end

function CritlineFilters_DeleteSpell(spellname, tree)
	for k, v in pairs(CL_FILTERS[tree]) do
		if v == spellname then
			tremove(CL_FILTERS[tree], k)
			do break end
		end
	end
	CLFSpellScrollBar_Update()
	Critline_RebuildAllTooltips()
end

function CritlineFilters_AddSpell(spellname, tree)
	if spellname then
		if not CritlineFilters_IsSpellInFilter(spellname,tree) then
			tinsert(CL_FILTERS[tree], spellname)
			CLFSpellScrollBar_Update()
			Critline_RebuildAllTooltips()
		end
	end
end

function CritlineFilters_IsMobInFilter(mobName)
	for i, v in ipairs(CL_MOBFILTER_DEFAULTS) do
		if v == mobName then
			Critline_Debug("Name of Mob ("..mobName..") is in Filter ...")
			return true
		end
	end
	for i, v in ipairs(CL_MOBFILTERS["mobs"]) do
		if v == mobName then
			Critline_Debug("Name of Mob ("..mobName..") is in Filter ...")
			return true
		end
	end
	Critline_Debug("Name of Mob ("..mobName..") is NOT in Filter ...")
	return false
end

function CritlineFilters_IsSpellInFilter(spellName, tree)
	for k, v in pairs(CL_FILTERS[tree]) do
		if v == spellName then
			Critline_Debug("Name of Spell ("..spellName..") is in Filter ...")
			return true
		end
	end
	Critline_Debug("Name of Spell ("..spellName..") is NOT in Filter ...")
	return false
end

function CritlineFilters_IsSpellInSpellbook(spellname)
	local numspells = 1
	while GetSpellName(numspells, BOOKTYPE_SPELL) do --find the top rank of every spell
		local name = GetSpellName(numspells, BOOKTYPE_SPELL)
		if name == spellname then
			return true
		end
		numspells = numspells + 1
	end
	return false
end

function CritlineFiltersButton_OnClick(self)
	self:LockHighlight()
	if strmatch(self:GetName(), "Mob") then
		CLFMobScrollBar.selected = self:GetID() + FauxScrollFrame_GetOffset(CLFMobScrollBar)
	else
		CLFSpellScrollBar.selected = self:GetID() + FauxScrollFrame_GetOffset(CLFSpellScrollBar)
	end
end

function CritlineSettings_SpellButton_OnClick(self)
	local id = self:GetID()
	if self:GetChecked() then
		CritlineFilters_AddSpell(_G["CLFSpellButton"..id.."Text"]:GetText(), CLF_tree)
	else
		CritlineFilters_DeleteSpell(_G["CLFSpellButton"..id.."Text"]:GetText(), CLF_tree)
	end
end

function CritlineFilters_Localize()
	Critline_FiltersTitleString:SetText(CL_FILTERS_TEXT)
	CritlineFilters_MobTitle:SetText(CL_MOBFILTER_TEXT)
	CritlineFilters_SpellTitle:SetText(CL_SPELLFILTER_TEXT)
	CritlineFilters_AddSpellText:SetText(CL_ADDSPELL_TEXT)
	Critline_FiltersButton4:SetText(CL_ADDTARGET_TEXT)
	Critline_FiltersButton1:SetText(CL_DELETEMOB_TEXT)
end